/**
  ******************************************************************************
  * @file      fab_nfo.h
  * @author    Marek Laban, Marcel Kleja, MICRONIC a.s., LabHC
  * @version   v1.0
  * @note      Created in frame of HECTOR project (No. 644052).
  * @copyright MICRONIC a.s., LabHC
  * @date      16-May-2016
  * @revision  v1.0 created (ML)
  * @brief     Contains registers and values used at FABRIC control block
  *            (ctrl.vhd)
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef FAB_NFO_H_
#define FAB_NFO_H_

/** @defgroup FPGA_register_map
  * @{
  */
#define CMD_ADDR     0x70000000		/**< Command address                      */
#define STATE_ADDR   0x70000004     /**< Acq State address                    */
#define STAGE_ADDR   0x70000008     /**< Actual ram interface stage           */
#define FABIN_ADDR   0x7000000C     /**< Fabric input register (from MSS)     */
#define FABOUT_ADDR  0x70000010	    /**< Fabric output register (to MSS)      */
#define FABST_ADDR   0x70000014	    /**< Fabric status register (to MSS)      */
#define WSTART_ADDR  0x70000018     /**< Write to RAM interface start address */
#define WSIZE_ADDR   0x7000001C     /**< Write to RAM interface stream size   */
#define RSTART_ADDR  0x70000020     /**< Read from RAM interface start address*/
#define RSIZE_ADDR   0x70000024     /**< Read from RAM interface stream size  */
#define USER1_ADDR   0x70000028     /**< User optional register address       */
#define USER2_ADDR   0x7000002C     /**< User optional register address       */

/** @defgroup Acquisition_states
  * @{
  */
#define OP_IDLE_STATE 0xFFFFFFFF	/**< idle state                 */
#define OP_BUSY_STATE 0x00000001	/**< busy state                 */
#define OP_OFLW_STATE 0x00000002    /**< FIFO buffer overflow state */
#define OP_ERR_STATE  0x00000003    /**< Other error of operation   */


/** @defgroup Commands
  * @{
  */
#define CMD_OP_IDLE 0x00000000 /**< operation start command */
#define CMD_OP_ST   0x00000001 /**< operation start command */
#define CMD_SETDB1  0x00000002 /**< set daughter board 1    */
#define CMD_SETDB2  0x00000003 /**< set daughter board 2    */
// user defined commands
#define CMD_USER0   0x00000004 /**< user command 1          */
#define CMD_USER1   0x00000005 /**< user command 2          */

#endif /* FAB_NFO_H_ */
