-------------------------------------------------------------------------------
--
-- Design unit:   Multiplexer 
--                 
--
-- File name:     mux.vhd
--
-- Description:   Multiplex input signals for system between 
--                HDMI and SATA connector 
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, MICRONIC a.s., LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------

LIBRARY IEEE;

USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY mux IS
PORT (
  -- data
	data_in_1 : IN  std_logic; 
	data_in_2 : IN  std_logic; 
    data_out  : OUT std_logic; 
  -- data clock
	clk_in_1  : IN  std_logic; 
	clk_in_2  : IN  std_logic; 
    clk_out   : OUT std_logic; 
  -- synchronization of 32-bit words
    sync_in_1 : IN  std_logic;
    sync_in_2 : IN  std_logic;
    sync_out  : OUT std_logic;
  -- multiplexor config
    set_mux   : IN  std_logic 

);
END mux;

ARCHITECTURE architecture_mux OF mux IS
BEGIN

  WITH set_mux SELECT                             
      data_out <=  data_in_1   WHEN '0',
                   data_in_2   WHEN '1',
                   '0'  WHEN OTHERS;

  WITH set_mux SELECT                             
      clk_out <=  clk_in_1   WHEN '0',
                  clk_in_2   WHEN '1',
                  '0'  WHEN OTHERS;

  WITH set_mux SELECT                             
      sync_out <=  sync_in_1   WHEN '0',
                   sync_in_2   WHEN '1',
                  '0'  WHEN OTHERS;

END architecture_mux;
