# Load the HECTOR tcl extension
source HECTOR_data_acq.tcl

# Open the serial device
set dev [openDevice COM3]

# Reset the mohterboard
softReset $dev

# Create a filesystem in the RAM
createFileSystem $dev

# Mount the filesystem to the PC
mountDisk $dev

# Find out the drive letter (mountpoint) of the HECTOR disk
set hectorDrive [findHECTOR]

# Fail if the disk is not found
if {$hectorDrive == 0} {
  puts "HECTOR disk not found"
  return -1
}

# Copy the input file onto the HECTOR disk
file copy -force "./input.txt" "[lindex $hectorDrive 2]/input.txt"

# Synchronize the filesystems
syncDrives

# Load the input data from file to the cache RAM
loadInputFile $dev input.txt 33324200

# Create a new filesystem to format it
createFileSystem $dev

# Set the operation mode
#   4 = convert to lowercase
#   5 = convert to uppercase
sendFabricCommand $dev 5 0

# Start the conversion and save the result to file
beginAcquisition $dev output.txt 33324200

# Check for the status
set err [getStatus $dev]
while {[lindex $err 2] != 0} {
  set err [getStatus $dev]

  puts -nonewline "[lindex $err 1] %\r"
  flush stdout
}
puts {}
flush stdout

# Mount the HECTOR disk to the PC
mountDisk $dev

# Find out the drive letter (mountpoint) of the HECTOR disk
set hectorDrive [findHECTOR]

# Fail if the disk is not found
if {$hectorDrive == 0} {
  puts "HECTOR disk not found"
  return -1
}

# Copy the output file to the PC
file copy -force "[lindex $hectorDrive 2]/output.txt" "output.txt"

# Close the connection to the board
disconnect $dev
