# Load the HECTOR tcl extension
source HECTOR_data_acq.tcl

# Open the serial device
set dev [openDevice COM3]

for {set i 0} {$i < 1} {incr i} {
  
  puts "Measurement # $i"

# Reset the mohterboard
  softReset $dev

# Select the daughter board connection (1 = HDMI, 2 = SATA)
  selectDaughterBoard $dev 2

# Enable (4) or disable (5) word synchronization
  sendFabricCommand $dev 4 0

# Create filesystem for data
  createFileSystem $dev

# Begin data acquisition
#     We must provide the filename that will be assigned
#     to acquired data file. Last parameter sets the size
#     of the file in bytes.
  beginAcquisition $dev ACQ.DAT 1024

# Wait until the acquisition is finished
  set err [getStatus $dev]
  while {[lindex $err 2] != 0} {
    set err [getStatus $dev]

# Print progress
    puts -nonewline "[lindex $err 1] %\r"
    flush stdout
  }
  puts {}
  flush stdout

# Mount the disk to make it available for the PC
  mountDisk $dev

# Automatically detect the HECTOR disk
  set hectorDrive [findHECTOR]

  if { $hectorDrive == 0 } {
    puts "Hector disk drive not found. You will have to search for it manually."
    return 1
  }

# Check for the mountpoint
  if { [string length [lindex $hectorDrive 2]] == 0 } {
    puts "Hector disk drive is not mounted. Please mount the device [lindex $hectorDrive 0]."
    return 1
  }

# Copy the data from HECTOR disk
  file copy -force "[lindex $hectorDrive 2]/ACQ.DAT" "./cnt_$i.dat"
}

# Close the connection to the board
disconnect $dev
