# Load the HECTOR tcl extension
source HECTOR_data_acq.tcl

# Open the serial device
set dev [openDevice COM3]

for {set i 0} {$i < 1} {incr i} {
  
  puts "Measurement # $i"

# Reset the mohterboard
  softReset $dev

# Select the daughter board connection (1 = HDMI, 2 = SATA)
  selectDaughterBoard $dev 2

# Configure the GPIOs as outputs
#     second parameter is the GPIO configuration
#     which is an 8-bit word where every bit corresponds
#     to one GPIO. 1 is output and 0 input
  configureGPIO $dev 255

# Set the GPIOs to configure the TRNG output multiplexer
#   0 = XOR - TRNG output after decimator
#   1 = DFF - raw random data before decimator
  setGPIO $dev 0

#Synchronize the filesystems
syncDrives
  
# Create filesystem for data
  createFileSystem $dev

# Reset daughter board
  sendDaughterReset $dev

# Begin data acquisition
#     We must provide the filename that will be assigned
#     to acquired data file. Last parameter sets the size
#     of the file in bytes.
  beginAcquisition $dev ACQ.DAT 125000

# Wait until the acquisition is finished
  set err [getStatus $dev]
  while {[lindex $err 2] != 0} {
    set err [getStatus $dev]

# Print progress
    puts -nonewline "[lindex $err 1] %\r"
    flush stdout
  }
  puts {}
  flush stdout
  
# Mount the disk to make it available for the PC
  mountDisk $dev

# Automatically detect the HECTOR disk
  set hectorDrive [findHECTOR]

  if { $hectorDrive == 0 } {
    puts "Hector disk drive not found. You will have to search for it manually."
    return 1
  }

# Check for the mountpoint
  if { [string length [lindex $hectorDrive 2]] == 0 } {
    puts "Hector disk drive is not mounted. Please mount the device [lindex $hectorDrive 0]."
    return 1
  }

# Copy the data from HECTOR disk
  file copy -force "[lindex $hectorDrive 2]/ACQ.DAT" "./acq_$i.bin"
}

# Close the connection to the board
disconnect $dev
