# Load the HECTOR tcl extension
source HECTOR_data_acq.tcl

# Open the serial device
set dev [openDevice COM3]

# Reset the mohterboard
softReset $dev

# Select the daughter board connection (1 = HDMI, 2 = SATA)
selectDaughterBoard $dev 2

# Configure the GPIOs as outputs
#     second parameter is the GPIO configuration
#     which is an 8-bit word where every bit corresponds
#     to one GPIO. 1 is output and 0 input
configureGPIO $dev 255

# Set the GPIOs to configure the TRNG output multiplexer
#   0 = XOR - TRNG output after decimator
#   1 = DFF - raw random data before decimator
setGPIO $dev 0

# Create filesystem for data
createFileSystem $dev

# Reset daughter board
sendDaughterReset $dev

# Begin data acquisition
#     We must provide the filename that will be assigned
#     to acquired data file. Last parameter sets the size
#     of the file in bytes.
beginAcquisition $dev acq.bin 125000

# Wait until the acquisition is finished
set err [getStatus $dev]
while {[lindex $err 2] != 0} {
  set err [getStatus $dev]
}

# Mount the disk to make it available for the PC
mountDisk $dev

# Close the connection to the board
disconnect $dev
